@@ -2,37 +2,38 @@ from __future__ import annotations  # todo0 remove when it's by default
 
 __all__ = ['DiscordBot']
 
-import asyncio
+import contextlib
 import datetime
 import io
+import logging
 import pathlib
 import random
 import traceback
-from typing import Any, Iterable, Sequence
+from collections.abc import Iterable, Sequence
+from typing import Any
 
 import discord
+import discord.ext
 import flanautils
-import pymongo
-from discord.ext.commands import Bot
-from flanautils import Media, MediaType, NotFoundError, OrderedSet, return_if_first_empty
+from flanautils import Media, MediaType, NotFoundError, OrderedSet, ResponseError, return_if_first_empty
 
 from multibot import constants
 from multibot.bots.multi_bot import MultiBot, parse_arguments
 from multibot.exceptions import LimitError, SendError, UserDisconnectedError
-from multibot.models import Button, ButtonsInfo, Chat, Message, Mute, Platform, Role, User
+from multibot.models import Button, Chat, Message, Mute, Platform, Role, User
 
 
 # --------------------------------------------------------------------------------------------------- #
 # ------------------------------------------- DISCORD_BOT ------------------------------------------- #
 # --------------------------------------------------------------------------------------------------- #
-class DiscordBot(MultiBot[Bot]):
-    def __init__(self, bot_token: str):
-        super().__init__(bot_token=bot_token,
-                         bot_client=Bot(command_prefix=constants.DISCORD_COMMAND_PREFIX, intents=discord.Intents.all()))
+class DiscordBot(MultiBot[discord.ext.commands.Bot]):
+    def __init__(self, token: str):
+        super().__init__(token=token,
+                         client=discord.ext.commands.Bot(command_prefix=constants.DISCORD_COMMAND_PREFIX, intents=discord.Intents.all()))
 
-    # ----------------------------------------------------------- #
-    # -------------------- PROTECTED METHODS -------------------- #
-    # ----------------------------------------------------------- #
+    # -------------------------------------------------------- #
+    # ------------------- PROTECTED METHODS ------------------ #
+    # -------------------------------------------------------- #
     def _add_handlers(self):
         super()._add_handlers()
         self.client.add_listener(self._on_ready, 'on_ready')
@@ -82,7 +83,7 @@ class DiscordBot(MultiBot[Bot]):
         except AttributeError:
             current_roles = OrderedSet()
 
-        if database_user_data := self.User.collection.find_one({'platform': self.platform.value, 'id': original_user.id}):
+        if database_user_data := self.User.find_one_raw({'platform': self.platform.value, 'id': original_user.id}):
             database_roles = OrderedSet(Role.find({'_id': {'$in': database_user_data['roles']}}))
         else:
             database_roles = OrderedSet()
@@ -172,8 +173,8 @@ class DiscordBot(MultiBot[Bot]):
         return original_message.id
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
-    async def _get_original_message(self, event: constants.DISCORD_EVENT) -> discord.Message:
-        if isinstance(event, discord.Interaction):
+    async def _get_original_message(self, event: constants.DISCORD_EVENT) -> constants.DISCORD_MESSAGE:
+        if isinstance(event, constants.DISCORD_BUTTON_EVENT):
             return event.message
         else:
             return event
@@ -223,7 +224,8 @@ class DiscordBot(MultiBot[Bot]):
         if not media:
             return
 
-        file_name = f"{media.title or 'bot_media'}.{media.extension}"
+        file_stem = media.title or 'bot_media'
+        file_name = f"{file_stem}{f'.{media.extension}' if media.extension else ''}"
 
         if media.url:
             if pathlib.Path(media.url).is_file():
@@ -232,15 +234,27 @@ class DiscordBot(MultiBot[Bot]):
                 else:
                     return discord.File(media.url, filename=file_name)
             elif not media.bytes_:
-                media.bytes_ = await flanautils.get_request(media.url)
+                try:
+                    media.bytes_ = await flanautils.get_request(media.url)
+                except ResponseError:
+                    pass
 
         if bytes_ := media.bytes_:
             if media.type_ is MediaType.GIF:
                 bytes_ = await flanautils.to_gif(bytes_)
+            if media.title:
+                try:
+                    bytes_ = await flanautils.edit_metadata(bytes_, {'title': file_stem}, overwrite=False)
+                except FileNotFoundError:
+                    pass
             if len(bytes_) > constants.DISCORD_MEDIA_MAX_BYTES:
                 raise LimitError
             return discord.File(fp=io.BytesIO(bytes_), filename=file_name)
 
+    async def _start_async(self):
+        async with self.client:
+            await self.client.start(self.token)
+
     async def _unban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         user = await self.get_user(user)
         discord_group = await self._get_discord_group(group_)
@@ -261,6 +275,7 @@ class DiscordBot(MultiBot[Bot]):
         self.id = self.client.user.id
         self.name = self.client.user.name
         self.owner_id = (await self.client.application_info()).owner.id
+        discord.utils.setup_logging(level=logging.ERROR)
         await super()._on_ready()
 
     # -------------------------------------------------------- #
@@ -294,8 +309,10 @@ class DiscordBot(MultiBot[Bot]):
         if n_messages > 100:
             raise LimitError('El máximo es 100.')
 
-        chat = await self.get_chat(chat)
-        n_messages += 1
+        if (chat := await self.get_chat(chat)).is_private:
+            return
+
+        n_messages = int(n_messages)
 
         while n_messages > 0:
             if n_messages > 100:
@@ -306,14 +323,14 @@ class DiscordBot(MultiBot[Bot]):
                 n_messages = 0
 
             try:
-                await chat.original_object.purge(limit=n_messages_chunk)
+                message_ids = [message.id for message in await chat.original_object.purge(limit=n_messages_chunk)]
             except discord.errors.HTTPException:
                 raise LimitError(f'Solo puedo eliminar mensajes con menos de 14 días {random.choice(constants.SAD_EMOJIS)}')
 
-            database_messages_to_delete_generator = self.Message.find({'platform': self.platform.value, 'chat': chat.object_id}, sort_keys=(('date', pymongo.DESCENDING),), limit=n_messages_chunk, lazy=True)
-            for database_message_to_delete in database_messages_to_delete_generator:
-                database_message_to_delete.is_deleted = True
-                database_message_to_delete.save()
+            deleted_messages_generator = self.Message.find({'platform': self.platform.value, 'id': {'$in': message_ids}, 'chat': chat.object_id}, lazy=True)
+            for deleted_message in deleted_messages_generator:
+                deleted_message.is_deleted = True
+                deleted_message.save()
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
     async def delete_message(
@@ -340,7 +357,7 @@ class DiscordBot(MultiBot[Bot]):
             return
 
         message_to_delete.is_deleted = True
-        message_to_delete.save()
+        message_to_delete.save(('is_deleted',))
 
     # noinspection PyTypeChecker
     def distribute_buttons(self, texts: Sequence[str], vertically=False) -> list[list[str]]:
@@ -404,7 +421,10 @@ class DiscordBot(MultiBot[Bot]):
             case int(chat_id):
                 pass
             case str(chat_name):
-                chat_id = self.Chat.find_one({'platform': self.platform.value, 'name': chat_name}).id
+                if chat := self.Chat.find_one({'platform': self.platform.value, 'name': chat_name}):
+                    chat_id = chat.id
+                else:
+                    return
             case self.User() as user:
                 return await self._create_chat_from_discord_chat(await user.original_object.create_dm())
             case self.Chat():
@@ -414,14 +434,17 @@ class DiscordBot(MultiBot[Bot]):
             case _:
                 raise TypeError('bad arguments')
 
-        # noinspection PyTypeChecker
-        return await self._create_chat_from_discord_chat(self.client.get_channel(chat_id) or await self.client.fetch_channel(chat_id))
+        try:
+            # noinspection PyTypeChecker
+            return await self._create_chat_from_discord_chat(self.client.get_channel(chat_id) or await self.client.fetch_channel(chat_id))
+        except discord.errors.NotFound:
+            pass
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
     async def get_current_roles(self, user: int | str | User | constants.DISCORD_USER, group_: int | str | Chat | Message = None) -> list[Role]:
         if isinstance(user, User) and not group_:
             original_user = user.original_object
-        elif isinstance(user, (int, str, User)):
+        elif isinstance(user, int | str | User):
             original_user = (await self.get_user(user, group_)).original_object
         elif isinstance(user, constants.DISCORD_USER):
             original_user = user
@@ -443,7 +466,10 @@ class DiscordBot(MultiBot[Bot]):
             return await self.get_user(user, group_)
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
-    async def get_message(self, chat: int | str | User | Chat | Message, message: int | str | Message) -> Message | None:
+    async def get_message(self, message: int | str | Message, chat: int | str | User | Chat | Message) -> Message | None:
+        if not (chat := await self.get_chat(chat)):
+            return
+
         match message:
             case int(message_id):
                 pass
@@ -454,8 +480,10 @@ class DiscordBot(MultiBot[Bot]):
             case _:
                 raise TypeError('bad arguments')
 
-        chat = await self.get_chat(chat)
-        return await self._get_message(await chat.original_object.fetch_message(message_id))
+        try:
+            return await self._get_message(await chat.original_object.fetch_message(message_id))
+        except discord.errors.NotFound:
+            return
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
     async def get_group_roles(self, group_: int | str | Chat | Message) -> list[Role]:
@@ -467,13 +495,16 @@ class DiscordBot(MultiBot[Bot]):
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
     async def get_user(self, user: int | str | User, group_: int | str | Chat | Message = None) -> User | None:
-        user_id = self.get_user_id(user)
+        if not (user_id := self.get_user_id(user)):
+            return
 
-        if group_ is None:
-            original_user = self.client.get_user(user_id) or await self.client.fetch_user(user_id)
-        else:
-            discord_group = await self._get_discord_group(group_)
-            original_user = discord_group.get_member(user_id)
+        try:
+            if group_ is None:
+                original_user = self.client.get_user(user_id) or await self.client.fetch_user(user_id)
+            elif not (discord_group := await self._get_discord_group(group_)) or not (original_user := discord_group.get_member(user_id)):
+                return
+        except discord.errors.NotFound:
+            return
 
         return await self._create_user_from_discord_user(original_user)
 
@@ -524,6 +555,16 @@ class DiscordBot(MultiBot[Bot]):
         except AttributeError:
             raise UserDisconnectedError(user.name)
 
+    async def make_mention(self, user: int | str | User, group_: int | str | Chat | Message = None) -> str:
+        if isinstance(user, int):
+            id = user
+        else:
+            if isinstance(user, str):
+                user = await self.get_user(user, group_)
+            id = user.id
+
+        return f'<@{id}>'
+
     async def remove_role(self, user: int | str | User, group_: int | str | Chat | Message, role: int | str | Role):
         user = await self.get_user(user, group_)
         role = await self.find_role(role, group_)
@@ -558,19 +599,33 @@ class DiscordBot(MultiBot[Bot]):
     ) -> Message | None:
         async def file_too_large():
             if random.randint(0, 10):
-                error_message = 'El archivo pesa más de 8 MB.'
+                error_message = f'El archivo pesa más de {constants.DISCORD_MEDIA_MAX_BYTES // 1000000} MB.'
             else:
                 error_message = 'El archivo pesa mas que tu madre'
             await self._manage_exceptions(SendError(error_message), chat)
 
         text = self._parse_html_to_discord_markdown(text)
-        try:
-            file = await self._prepare_media_to_send(media)
-        except LimitError:
-            await file_too_large()
-            return
-        view = None
+        if (
+                media
+                and
+                media.url
+                and
+                media.type_ is MediaType.GIF
+                and
+                any(domain.lower() in media.url for domain in constants.GIF_DOMAINS)
+        ):
+            text = f'{text} {media.url}' if text else media.url
+            file = None
+        else:
+            try:
+                file = await self._prepare_media_to_send(media)
+            except LimitError:
+                await file_too_large()
+                return
+            if not text and not file:
+                return
 
+        view = None
         if buttons:
             view = discord.ui.View(timeout=None)
             for i, row in enumerate(buttons):
@@ -587,28 +642,13 @@ class DiscordBot(MultiBot[Bot]):
                 kwargs['attachments'] = [file]
             if buttons is not None:
                 kwargs['view'] = view
-                message.buttons_info.buttons = buttons
-            if buttons_key is not None:
-                message.buttons_info.key = buttons_key
 
             try:
                 message.original_object = await message.original_object.edit(**kwargs)
             except discord.errors.NotFound:
                 return
 
-            if data is None:
-                if media is not None:
-                    message.data['media'] = media.content
-            else:
-                if media is None:
-                    message.data = data
-                else:
-                    message.data = {'media': media.content} | data
-
-            message.update_last_edit()
-            message.save()
-
-            return message
+            return self._update_message_attributes(message, media, buttons, chat, buttons_key, data, update_last_edit=True)
 
         match reply_to:
             case int(message_id):
@@ -619,33 +659,15 @@ class DiscordBot(MultiBot[Bot]):
                 reply_to = message_to_reply.original_object
 
         try:
-            bot_message = await self._get_message(await chat.original_object.send(text, file=file, view=view, reference=reply_to))
+            bot_message = await self._get_message(await chat.original_object.send(text, file=file, view=view, reference=reply_to, silent=silent))
         except discord.errors.HTTPException as e:
             if 'too large' in str(e).lower():
                 await file_too_large()
                 return
             raise
 
-        bot_message.buttons_info = ButtonsInfo(buttons=buttons, key=buttons_key)
-        if media:
-            bot_message.data['media'] = media.content
-        if data:
-            bot_message.data |= data
+        return self._update_message_attributes(bot_message, media, buttons, chat, buttons_key, data)
 
-        bot_message.save()
-
-        return bot_message
-
-    def start(self):
-        async def start_():
-            await self.client.start(self.token)
-
-        try:
-            asyncio.get_running_loop()
-            return start_()
-        except RuntimeError:
-            asyncio.run(start_())
-
-    async def typing_delay(self, message: Message):
-        async with message.chat.original_object.typing():
-            await asyncio.sleep(random.randint(1, 3))
+    async def typing(self, chat: int | str | User | Chat | Message) -> contextlib.AbstractAsyncContextManager:
+        chat = await self.get_chat(chat)
+        return chat.original_object.typing()
