@@ -9,38 +9,41 @@ __all__ = [
     'ignore_self_message',
     'inline',
     'out_of_service',
+    'owner',
     'parse_arguments',
     'reply',
     'MultiBot'
 ]
 
+import asyncio
+import contextlib
 import datetime
 import functools
 import random
 import traceback
-from abc import ABC, abstractmethod
-from typing import Any, Callable, Generic, Iterable, Sequence, Type, TypeVar, overload
+from abc import ABC
+from collections import defaultdict
+from collections.abc import Callable, Coroutine, Iterable, Iterator, Sequence
+from typing import Any, Generic, Literal, TypeVar, overload
 
-import discord
 import flanautils
-import telethon.events
-import telethon.events.common
+import pymongo
 from flanautils import AmbiguityError, Media, NotFoundError, OrderedSet, ScoreMatch, return_if_first_empty, shift_args_if_called
 
 from multibot import constants
 from multibot.exceptions import BadRoleError, LimitError, SendError, UserDisconnectedError
-from multibot.models import Ban, Button, Chat, Message, Mute, Penalty, Platform, RegisteredButtonCallback, RegisteredCallback, Role, User
+from multibot.models import Ban, Button, ButtonsInfo, Chat, Message, Mute, Penalty, Platform, RegisteredCallback, Role, User
 
 
-# ---------------------------------------------------------- #
-# ----------------------- DECORATORS ----------------------- #
-# ---------------------------------------------------------- #
+# ---------------------------------------------------- #
+# -------------------- DECORATORS -------------------- #
+# ---------------------------------------------------- #
 @shift_args_if_called
 def find_message(func_: Callable = None, /, return_if_not_found=False) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         async def wrapper(self: MultiBot, *args, **kwargs):
-            def take_arg(type_: Type, args_, kwargs_):
+            def take_arg(type_: type, args_, kwargs_):
                 object_ = None
                 new_args = []
                 for arg in args_:
@@ -79,7 +82,6 @@ def admin(func_: Callable = None, /, is_=True, send_negative=False) -> Callable:
         @functools.wraps(func)
         @find_message
         async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
-            message = message
             if is_ is message.author.is_admin or message.chat.is_private:
                 return await func(self, message, *args, **kwargs)
             await self.accept_button_event(message)
@@ -168,6 +170,23 @@ def out_of_service(func: Callable) -> Callable:
     return wrapper
 
 
+@shift_args_if_called
+def owner(func_: Callable = None, /, is_=True, send_negative=False) -> Callable:
+    def decorator(func: Callable) -> Callable:
+        @functools.wraps(func)
+        @find_message
+        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
+            if is_ is (message.author.id == self.owner_id):
+                return await func(self, message, *args, **kwargs)
+            await self.accept_button_event(message)
+            if send_negative:
+                await self.send_negative(message)
+
+        return wrapper
+
+    return decorator(func_) if func_ else decorator
+
+
 def parse_arguments(func: Callable) -> Callable:
     @functools.wraps(func)
     async def wrapper(*args, **kwargs) -> Any:
@@ -278,33 +297,34 @@ T = TypeVar('T')
 
 
 class MultiBot(Generic[T], ABC):
-    client: T
     Chat = Chat
     Message = Message
     User = User
 
-    def __init__(self, bot_token: str, bot_client: T):
+    def __init__(self, token: str, client: T):
         self.platform: Platform | None = None
         self.id: int | None = None
         self.name: str | None = None
         self.owner_id: int | None = None
-        self.token: str = bot_token
-        self.client: T = bot_client
+        self._owner_chat: Chat | None = None
+        self.token: str = token
+        self.client: T = client
         self._registered_callbacks: list[RegisteredCallback] = []
-        self._registered_button_callbacks: list[RegisteredButtonCallback] = []
+        self._registered_button_callbacks: dict[Any, list[Callable]] = defaultdict(list)
+        # noinspection PyPep8Naming
+        MessageType = self.Message
+        self._message_cache: dict[tuple[int, int], MessageType] = {}
 
-        self._add_handlers()
-
-    # ----------------------------------------------------------- #
-    # -------------------- PROTECTED METHODS -------------------- #
-    # ----------------------------------------------------------- #
+    # -------------------------------------------------------- #
+    # ------------------- PROTECTED METHODS ------------------ #
+    # -------------------------------------------------------- #
     def _add_handlers(self):
         pass
 
     async def _ban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
 
-    async def _check_penalties(self, penalty_class: Type[Penalty], unpenalize_method: Callable):
+    async def _check_penalties(self, penalty_class: type[Penalty], unpenalize_method: Callable):
         penalties = penalty_class.find({'platform': self.platform.value}, lazy=True)
 
         for penalty in penalties:
@@ -330,7 +350,6 @@ class MultiBot(Generic[T], ABC):
 
         return users - bot_user
 
-    @abstractmethod
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def _get_author(self, original_message: constants.ORIGINAL_MESSAGE) -> User | None:
         pass
@@ -343,12 +362,10 @@ class MultiBot(Generic[T], ABC):
     async def _get_button_presser_user(self, event: constants.MESSAGE_EVENT) -> User | None:
         pass
 
-    @abstractmethod
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def _get_chat(self, original_message: constants.ORIGINAL_MESSAGE) -> Chat | None:
         pass
 
-    @abstractmethod
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def _get_mentions(self, original_message: constants.ORIGINAL_MESSAGE) -> list[User]:
         pass
@@ -357,61 +374,69 @@ class MultiBot(Generic[T], ABC):
     async def _get_message(
         self,
         event: constants.MESSAGE_EVENT,
-        pull_overwrite_fields: Iterable[str] = ('_id',)
+        pull_overwrite_fields: Iterable[str] = ('_id', 'date')
     ) -> Message:
-        original_message = event if isinstance(event, constants.ORIGINAL_MESSAGE) else await self._get_original_message(event)
+        original_message = await self._get_original_message(event)
 
-        message = self.Message(
-            platform=self.platform,
-            id=await self._get_message_id(original_message),
-            author=await self._get_author(original_message),
-            text=await self._get_text(original_message),
-            mentions=await self._get_mentions(original_message),
-            chat=await self._get_chat(original_message),
-            replied_message=await self._get_replied_message(original_message),
-            is_inline=isinstance(event, telethon.events.InlineQuery.Event) if isinstance(event, constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE) else None,
-            original_object=original_message,
-            original_event=event
-        )
-        message.resolve()
-        message.pull_from_database()
-        if message.buttons_info:
-            message.buttons_info.pressed_text = await self._get_button_pressed_text(event)
-            message.buttons_info.presser_user = await self._get_button_presser_user(event)
-        message.save(pull_overwrite_fields=pull_overwrite_fields)
+        message_id = await self._get_message_id(original_message)
+        chat = await self._get_chat(original_message)
+        try:
+            cached_message = self._message_cache[message_id, chat.id]
+        except KeyError:
+            message = self.Message(
+                platform=self.platform,
+                id=message_id,
+                author=await self._get_author(original_message),
+                text=await self._get_text(original_message),
+                mentions=await self._get_mentions(original_message),
+                chat=chat,
+                replied_message=await self._get_replied_message(original_message),
+                is_inline=isinstance(event, constants.TELEGRAM_INLINE_EVENT) if isinstance(event, constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE) else None,
+                original_object=original_message,
+                original_event=event
+            )
+            message.resolve()
+            message.save(pull_overwrite_fields=pull_overwrite_fields, pull_lazy=False)
+            return message
 
-        return message
+        if cached_message.buttons_info:
+            cached_message.buttons_info.pressed_text = await self._get_button_pressed_text(event)
+            cached_message.buttons_info.presser_user = await self._get_button_presser_user(event)
+        cached_message.original_object = original_message
+        cached_message.original_event = event
+        return cached_message
 
-    @abstractmethod
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def _get_message_id(self, original_message: constants.ORIGINAL_MESSAGE) -> int | str | None:
         pass
 
-    @abstractmethod
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def _get_original_message(self, event: constants.MESSAGE_EVENT) -> constants.ORIGINAL_MESSAGE:
         pass
 
-    @abstractmethod
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def _get_replied_message(self, original_message: constants.ORIGINAL_MESSAGE) -> Message | None:
         pass
 
-    @abstractmethod
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def _get_text(self, original_message: constants.ORIGINAL_MESSAGE) -> str:
         pass
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
-    async def _manage_exceptions(self, exceptions: BaseException | Iterable[BaseException], context: Chat | Message):
+    async def _manage_exceptions(
+        self,
+        exceptions: Exception | Iterable[Exception],
+        context: Chat | Message,
+        reraise=False,
+        print_traceback=False
+    ):
         if not isinstance(exceptions, Iterable):
             exceptions = (exceptions,)
 
         for exception in exceptions:
+            # noinspection PyBroadException
             try:
                 raise exception
-            except ValueError:
-                await self.delete_message(context)
             except LimitError as e:
                 await self.delete_message(context)
                 await self.send_error(str(e), context)
@@ -426,7 +451,10 @@ class MultiBot(Generic[T], ABC):
                 if constants.SEND_EXCEPTION_MESSAGE_LINES:
                     traceback_message = '\n'.join(traceback.format_exc().splitlines()[-constants.SEND_EXCEPTION_MESSAGE_LINES:])
                     await self.send(f'{random.choice(constants.EXCEPTION_PHRASES)}\n\n...\n{traceback_message}', context)
-                raise
+                if print_traceback:
+                    print(traceback.format_exc())
+                if reraise:
+                    raise
 
     async def _mute(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
@@ -435,15 +463,15 @@ class MultiBot(Generic[T], ABC):
     def _parse_callbacks(
         text: str,
         registered_callbacks: list[RegisteredCallback],
-        score_reward_exponent: float = constants.SCORE_REWARD_EXPONENT,
-        keywords_lenght_penalty: float = constants.KEYWORDS_LENGHT_PENALTY,
-        minimum_score_to_match: float = constants.MINIMUM_SCORE_TO_MATCH
+        score_reward_exponent: float = constants.PARSER_SCORE_REWARD_EXPONENT,
+        keywords_lenght_penalty: float = constants.PARSER_KEYWORDS_LENGHT_PENALTY,
+        minimum_score_to_match: float = constants.PARSER_MIN_SCORE_TO_MATCH
     ) -> OrderedSet[RegisteredCallback]:
         text = flanautils.remove_accents(text.lower())
 
         original_words = OrderedSet()
         for word in text.split():
-            if len(word) <= constants.MAX_WORD_LENGTH:
+            if len(word) <= constants.PARSER_MAX_WORD_LENGTH:
                 original_words.add(word)
         important_words = original_words - flanautils.CommonWords.get()
 
@@ -459,7 +487,7 @@ class MultiBot(Generic[T], ABC):
                 mached_keywords_groups = 0
                 total_score = 0
                 for keywords_group in registered_callback.keywords:
-                    important_words |= {original_text_word for original_text_word in original_words if flanautils.cartesian_product_string_matching(original_text_word, keywords_group, min_score=registered_callback.min_score)}
+                    important_words |= {original_word for original_word in original_words if flanautils.cartesian_product_string_matching(original_word, keywords_group, min_score=registered_callback.min_score)}
                     word_matches = flanautils.cartesian_product_string_matching(important_words, keywords_group, min_score=registered_callback.min_score)
                     score = sum((max(matches.values()) + 1) ** score_reward_exponent for matches in word_matches.values())
                     try:
@@ -507,6 +535,12 @@ class MultiBot(Generic[T], ABC):
                 penalty.pull_from_database()
                 penalty.delete()
 
+    async def _start_async(self):
+        pass
+
+    def _start_sync(self):
+        asyncio.run(self._start_async())
+
     async def _unban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
 
@@ -514,43 +548,74 @@ class MultiBot(Generic[T], ABC):
         pass
 
     async def _unpenalize_later(self, penalty: Penalty, unpenalize_method: Callable, message: Message = None):
-        if penalty.time and datetime.timedelta() <= penalty.time <= constants.TIME_THRESHOLD_TO_MANUAL_UNPUNISH:
-            await flanautils.do_later(penalty.time, self._remove_penalty, penalty, unpenalize_method, message)
+        if penalty.time and penalty.time <= constants.TIME_THRESHOLD_TO_MANUAL_UNPUNISH:
+            flanautils.do_later(penalty.time, self._remove_penalty, penalty, unpenalize_method, message)
+
+    def _update_message_attributes(
+        self,
+        message: Message,
+        media: Media = None,
+        buttons: list[str | tuple[str, bool] | Button | list[str | tuple[str, bool] | Button]] | None = None,
+        chat: int | str | User | Chat | Message | None = None,
+        buttons_key: Any = None,
+        data: dict = None,
+        update_last_edit=False
+    ):
+        if media is not None and (not media.bytes_ or len(media.bytes_) <= constants.PYMONGO_MEDIA_MAX_BYTES):
+            message.medias = [media]
+        try:
+            if buttons is not None:
+                self._message_cache[message.id, chat.id].buttons_info.buttons = buttons
+            if buttons_key is not None:
+                self._message_cache[message.id, chat.id].buttons_info.key = buttons_key
+        except (AttributeError, KeyError):
+            message.buttons_info = ButtonsInfo(buttons=buttons, key=buttons_key)
+        if data is not None:
+            message.data = data
+        if message.buttons_info or message.data is not None:
+            self._message_cache[message.id, chat.id] = message
+
+        if update_last_edit:
+            message.update_last_edit()
+        message.save()
+
+        return message
 
     # ---------------------------------------------- #
     #                    HANDLERS                    #
     # ---------------------------------------------- #
     @find_message
     async def _on_button_press_raw(self, message: Message):
-        if getattr(message.buttons_info, 'key', None) is None:
+        if not message.buttons_info:
             return
 
-        for registered_callback in self._registered_button_callbacks:
-            if registered_callback.key == message.buttons_info.key:
-                try:
-                    await registered_callback(message)
-                except Exception as e:
-                    await self._manage_exceptions(e, message)
+        for callback in self._registered_button_callbacks[message.buttons_info.key]:
+            try:
+                await callback(message)
+            except Exception as e:
+                await self._manage_exceptions(e, message, reraise=True)
 
     @ignore_self_message
     async def _on_new_message_raw(self, message: Message):
         try:
             registered_callbacks = self._parse_callbacks(message.text, self._registered_callbacks)
         except AmbiguityError as e:
-            await self._manage_exceptions(e, message)
+            await self._manage_exceptions(e, message, reraise=True)
         else:
             for registered_callback in registered_callbacks:
                 try:
                     await registered_callback(message)
                 except Exception as e:
-                    await self._manage_exceptions(e, message)
+                    await self._manage_exceptions(e, message, reraise=True)
 
     async def _on_ready(self):
-        flanautils.init_db()
+        constants.load_environment()
+        flanautils.init_database()
         print(f'{self.name} activado en {self.platform.name} (id: {self.id})')
-        await flanautils.do_every(constants.CLEAR_OLD_DATABASE_ITEMS_EVERY_SECONDS, self.clear_old_database_items)
-        await flanautils.do_every(constants.CHECK_MUTES_EVERY_SECONDS, self.check_bans)
-        await flanautils.do_every(constants.CHECK_MUTES_EVERY_SECONDS, self.check_mutes)
+        flanautils.do_every(constants.CHECK_OLD_CACHE_MESSAGES_EVERY_SECONDS, self.check_old_cache_messages)
+        flanautils.do_every(constants.CHECK_OLD_DATABASE_MESSAGES_EVERY_SECONDS, self.check_old_database_messages)
+        flanautils.do_every(constants.CHECK_PENALTIES_EVERY_SECONDS, self.check_bans)
+        flanautils.do_every(constants.CHECK_PENALTIES_EVERY_SECONDS, self.check_mutes)
 
     # -------------------------------------------------------- #
     # -------------------- PUBLIC METHODS -------------------- #
@@ -574,15 +639,25 @@ class MultiBot(Generic[T], ABC):
     async def check_mutes(self):
         await self._check_penalties(Mute, self._unmute)
 
+    def check_old_cache_messages(self):
+        keys_to_delete = []
+        for k, v in self._message_cache.items():
+            if datetime.datetime.now(datetime.timezone.utc) < v.date + constants.BUTTONS_INFOS_EXPIRATION_TIME:
+                break
+
+            keys_to_delete.append(k)
+
+        for key in keys_to_delete:
+            del self._message_cache[key]
+
+    def check_old_database_messages(self):
+        before_date = datetime.datetime.now(datetime.timezone.utc) - constants.DATABASE_MESSAGE_EXPIRATION_TIME
+        self.Message.delete_many_raw({'platform': self.platform.value, 'date': {'$lte': before_date}})
+
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def clear(self, n_messages: int, chat: int | str | Chat | Message):
         pass
 
-    @classmethod
-    async def clear_old_database_items(cls):
-        before_date = datetime.datetime.now(datetime.timezone.utc) - constants.MESSAGE_EXPIRATION_TIME
-        cls.Message.collection.delete_many({'date': {'$lte': before_date}})
-
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def delete_message(
         self,
@@ -641,7 +716,32 @@ class MultiBot(Generic[T], ABC):
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def get_current_roles(self, user: int | str | User | constants.ORIGINAL_USER, group_: int | str | Chat | Message = None) -> list[Role]:
-        pass
+        return []
+
+    def get_formatted_last_database_messages(self, n_messages: int, name_limit=10, platform_limit=10, chat_limit=10, text_limit=40, timezone=None, simple=False) -> str:
+        if simple:
+            title = f"       {'Usuario'[:name_limit]:<{name_limit}}  {'Texto'[:text_limit]:<{text_limit}}  {'Fecha':<12}"
+
+            def generator_():
+                for i, message in enumerate(self.get_last_database_messages(n_messages, lazy=True), start=1):
+                    name = message.author.name.split('#')[0]
+                    text = repr(message.text).replace('`', '').strip("'")
+                    date = message.date.astimezone(timezone).strftime('%d  %H:%M')
+                    yield f"{i:>4}.  {name[:name_limit]:<{name_limit}}  {text[:text_limit]:<{text_limit}}  {date}"
+        else:
+            title = f"       {'Usuario'[:name_limit]:<{name_limit}}  {'Plataforma'[:platform_limit]:<{platform_limit}}  {'Chat'[:chat_limit]:<{chat_limit}}  {'Texto'[:text_limit]:<{text_limit}}  {'Fecha':<20}"
+
+            def generator_():
+                for i, message in enumerate(self.get_last_database_messages(n_messages, lazy=True), start=1):
+                    name = message.author.name.split('#')[0]
+                    platform = Platform(message.platform).name
+                    chat = message.chat.name
+                    text = repr(message.text).replace('`', '').strip("'")
+                    date = message.date.astimezone(timezone).strftime('%d/%m/%Y  %H:%M:%S')
+                    yield f"{i:>4}.  {name[:name_limit]:<{name_limit}}  {platform[:platform_limit]:<{platform_limit}}  {chat[:chat_limit]:<{chat_limit}}  {text[:text_limit]:<{text_limit}}  {date}"
+
+        joined_text = '\n'.join(generator_())
+        return f"­<code><code><code>{title}\n\n{joined_text}</code></code></code>"
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     def get_group_id(self, group_: int | str | Chat | Message) -> int | None:
@@ -673,11 +773,22 @@ class MultiBot(Generic[T], ABC):
             case self.Message() as message:
                 return message.chat.group_name
 
+    @overload
+    def get_last_database_messages(self, n_messages: int, lazy: Literal[False] = False) -> list[Message]:
+        pass
+
+    @overload
+    def get_last_database_messages(self, n_messages: int, lazy: Literal[True] = False) -> Iterator[Message]:
+        pass
+
+    def get_last_database_messages(self, n_messages: int, lazy=False) -> Iterator[Message] | list[Message]:
+        return self.Message.find(sort_keys=(('date', pymongo.DESCENDING),), limit=n_messages, lazy=lazy)
+
     async def get_me(self, group_: int | str | Chat | Message = None) -> User | None:
         pass
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
-    async def get_message(self, chat: int | str | User | Chat | Message, message: int | str | Message) -> Message | None:
+    async def get_message(self, message: int | str | Message, chat: int | str | User | Chat | Message) -> Message | None:
         pass
 
     @return_if_first_empty([], exclude_self_types='MultiBot', globals_=globals())
@@ -688,7 +799,6 @@ class MultiBot(Generic[T], ABC):
     async def get_user(self, user: int | str | User, group_: int | str | Chat | Message = None) -> User | None:
         pass
 
-    @abstractmethod
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def get_users(self, group_: int | str | Chat | Message) -> list[User]:
         pass
@@ -734,30 +844,32 @@ class MultiBot(Generic[T], ABC):
     async def is_self_muted(self, user: int | str | User, group_: int | str | Chat | Message) -> bool:
         pass
 
+    async def make_mention(self, user: int | str | User, group_: int | str | Chat | Message = None) -> str:
+        pass
+
     async def mute(self, user: int | str | User, group_: int | str | Chat | Message, time: int | datetime.timedelta = None, message: Message = None):
         # noinspection PyTypeChecker
         mute = Mute(self.platform, self.get_user_id(user), self.get_group_id(group_), time)
-        try:
-            await self._mute(mute.user_id, mute.group_id)
-        except UserDisconnectedError as e:
-            if message and message.chat.original_object:
-                await self._manage_exceptions(e, message)
-            else:
-                raise e
-        else:
-            mute.save(pull_exclude_fields=('until',))
-            await self._unpenalize_later(mute, self._unmute, message)
+        await self._mute(mute.user_id, mute.group_id, message)
+        mute.save(pull_exclude_fields=('until',))
+        await self._unpenalize_later(mute, self._unmute, message)
+
+    @property
+    async def owner_chat(self) -> Chat:
+        if not self._owner_chat:
+            self._owner_chat = await self.get_chat(self.owner_id) or await self.get_chat(await self.get_user(self.owner_id))
+        return self._owner_chat
 
     @overload
-    def register(self, func_: Callable = None, keywords=(), priority: int | float = 1, min_score=constants.PARSE_CALLBACKS_MIN_SCORE_DEFAULT, always=False, default=False):
+    def register(self, func_: Callable = None, keywords=(), priority: int | float = 1, min_score=constants.PARSER_MIN_SCORE_DEFAULT, always=False, default=False):
         pass
 
     @overload
-    def register(self, keywords=(), priority: int | float = 1, min_score=constants.PARSE_CALLBACKS_MIN_SCORE_DEFAULT, always=False, default=False):
+    def register(self, keywords=(), priority: int | float = 1, min_score=constants.PARSER_MIN_SCORE_DEFAULT, always=False, default=False):
         pass
 
     @shift_args_if_called(exclude_self_types='MultiBot', globals_=globals())
-    def register(self, func_: Callable = None, keywords: str | Iterable[str | Iterable[str]] = (), priority: int | float = 1, min_score=constants.PARSE_CALLBACKS_MIN_SCORE_DEFAULT, always=False, default=False):
+    def register(self, func_: Callable = None, keywords: str | Iterable[str | Iterable[str]] = (), priority: int | float = 1, min_score=constants.PARSER_MIN_SCORE_DEFAULT, always=False, default=False):
         def decorator(func):
             self._registered_callbacks.append(RegisteredCallback(func, keywords, priority, min_score, always, default))
             return func
@@ -775,7 +887,7 @@ class MultiBot(Generic[T], ABC):
     @shift_args_if_called(exclude_self_types='MultiBot', globals_=globals())
     def register_button(self, func_: Callable = None, key: Any = None):
         def decorator(func):
-            self._registered_button_callbacks.append(RegisteredButtonCallback(func, key))
+            self._registered_button_callbacks[key].append(func)
             return func
 
         return decorator(func_) if func_ else decorator
@@ -783,7 +895,6 @@ class MultiBot(Generic[T], ABC):
     async def remove_role(self, user: int | str | User, group_: int | str | Chat | Message, role: int | str | Role):
         pass
 
-    @abstractmethod
     @parse_arguments
     async def send(
         self,
@@ -803,26 +914,33 @@ class MultiBot(Generic[T], ABC):
         pass
 
     @parse_arguments
-    async def send_error(self, *args, exceptions_to_capture: Type[BaseException] | Iterable[Type[BaseException]] = (), **kwargs) -> constants.ORIGINAL_MESSAGE:
+    async def send_error(self, *args, **kwargs) -> constants.ORIGINAL_MESSAGE:
         bot_message = await self.send(*args, **kwargs)
-        await flanautils.do_later(constants.ERROR_MESSAGE_DURATION, self.delete_message, bot_message, exceptions_to_capture=exceptions_to_capture or discord.errors.NotFound)
+        flanautils.do_later(constants.ERROR_MESSAGE_DURATION, self.delete_message, bot_message)
         return bot_message
 
     @inline
     async def send_inline_results(self, message: Message):
         pass
 
-    async def send_interrogation(self, message: Message) -> constants.ORIGINAL_MESSAGE:
-        return await self.send(random.choice(constants.INTERROGATION_PHRASES), message)
+    async def send_interrogation(self, chat: int | str | User | Chat | Message) -> constants.ORIGINAL_MESSAGE:
+        chat = await self.get_chat(chat)
+        return await self.send(random.choice(constants.INTERROGATION_PHRASES), chat)
 
-    async def send_negative(self, message: Message) -> constants.ORIGINAL_MESSAGE:
-        return await self.send(random.choice(constants.NO_PHRASES), message)
+    async def send_negative(self, chat: int | str | User | Chat | Message) -> constants.ORIGINAL_MESSAGE:
+        chat = await self.get_chat(chat)
+        return await self.send(random.choice(constants.NO_PHRASES), chat)
 
-    @abstractmethod
-    async def start(self):
-        pass
+    def start(self) -> Coroutine | None:
+        self._add_handlers()
+        try:
+            asyncio.get_running_loop()
+        except RuntimeError:
+            self._start_sync()
+        else:
+            return self._start_async()
 
-    async def typing_delay(self, message: Message):
+    async def typing(self, chat: int | str | User | Chat | Message) -> contextlib.AbstractAsyncContextManager:
         pass
 
     async def unban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
