@@ -3,30 +3,46 @@ from __future__ import annotations  # todo0 remove when it's by default
 __all__ = ['user_client', 'TelegramBot']
 
 import asyncio
+import contextlib
 import functools
 import io
 import pathlib
+import struct
+from collections.abc import Coroutine
 from typing import Any, Callable, Sequence
 
 import flanautils
 import pymongo
-import telethon.events.common
+import telethon
 import telethon.hints
-import telethon.tl.functions.channels
 import telethon.tl.types
-from flanautils import Media, MediaType, OrderedSet, Source, return_if_first_empty, shift_args_if_called
+from flanautils import Media, MediaType, OrderedSet, ResponseError, Source, return_if_first_empty, shift_args_if_called
 from telethon import TelegramClient
 from telethon.sessions import StringSession
 
 from multibot import constants
 from multibot.bots.multi_bot import MultiBot, find_message, inline, parse_arguments
 from multibot.exceptions import LimitError
-from multibot.models import Button, ButtonsInfo, Chat, Message, Platform, User
+from multibot.models import Button, Chat, Message, Platform, User
 
 
-# ---------------------------------------------------------- #
-# ----------------------- DECORATORS ----------------------- #
-# ---------------------------------------------------------- #
+# ---------------------------------------------------- #
+# ----------------- CONTEXT MANAGERS ----------------- #
+# ---------------------------------------------------- #
+@contextlib.asynccontextmanager
+async def use_user_client(self: TelegramBot):
+    if await self.client.is_bot():
+        await self.user_client.connect()
+
+    yield self.user_client
+
+    if await self.client.is_bot():
+        await self.user_client.disconnect()
+
+
+# ---------------------------------------------------- #
+# -------------------- DECORATORS -------------------- #
+# ---------------------------------------------------- #
 @shift_args_if_called
 def user_client(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
@@ -44,18 +60,19 @@ def user_client(func_: Callable = None, /, is_=True) -> Callable:
 # ------------------------------------------- TELEGRAM_BOT ------------------------------------------- #
 # ---------------------------------------------------------------------------------------------------- #
 class TelegramBot(MultiBot[TelegramClient]):
-    def __init__(self, api_id: int | str, api_hash: int | str, phone: int | str = None, bot_token: str = None, bot_session: str = None, user_session: str = None):
+    def __init__(self, api_id: int | str, api_hash: int | str, bot_token: str = None, bot_session: str = None, phone: int | str = None, user_session: str = None):
         self.api_id = api_id
         self.api_hash = api_hash
-        self.phone = phone
         self.bot_session = bot_session
+        self.phone = phone
         self.user_session = user_session
-        self.inline_call_index = 0
 
         if self.bot_session:
-            bot_client = TelegramClient(StringSession(bot_session), self.api_id, self.api_hash)
+            client = TelegramClient(StringSession(bot_session), self.api_id, self.api_hash)
+        elif bot_token:
+            client = TelegramClient('bot_session', self.api_id, self.api_hash)
         else:
-            bot_client = TelegramClient('bot_session', self.api_id, self.api_hash)
+            client = None
 
         if self.user_session:
             self.user_client = TelegramClient(StringSession(user_session), self.api_id, self.api_hash)
@@ -64,12 +81,12 @@ class TelegramBot(MultiBot[TelegramClient]):
         else:
             self.user_client = None
 
-        super().__init__(bot_token=bot_token,
-                         bot_client=bot_client)
+        super().__init__(token=bot_token,
+                         client=client)
 
-    # ----------------------------------------------------------- #
-    # -------------------- PROTECTED METHODS -------------------- #
-    # ----------------------------------------------------------- #
+    # -------------------------------------------------------- #
+    # ------------------- PROTECTED METHODS ------------------ #
+    # -------------------------------------------------------- #
     # noinspection PyTypeChecker
     def _add_handlers(self):
         super()._add_handlers()
@@ -77,32 +94,15 @@ class TelegramBot(MultiBot[TelegramClient]):
         self.client.add_event_handler(self._on_inline_query_raw, telethon.events.InlineQuery)
         self.client.add_event_handler(self._on_new_message_raw, telethon.events.NewMessage)
 
-    @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
-    async def _create_bot_message_from_telegram_bot_message(
-        self,
-        original_message: constants.TELEGRAM_MESSAGE,
-        media: Media,
-        chat: Chat,
-        buttons: list[list[Button]] = None,
-        buttons_key: Any = None,
-        data: dict = None
-    ) -> Message | None:
-        original_message._sender = await self.client.get_entity(self.id)
-        original_message._chat = chat.original_object
-        bot_message = await self._get_message(original_message)
-        bot_message.buttons_info = ButtonsInfo(buttons=buttons, key=buttons_key)
-        if media and media.bytes_ and len(media.bytes_) <= constants.PYMONGO_MEDIA_MAX_BYTES:
-            bot_message.data['media'] = media.content
-        if data:
-            bot_message.data |= data
+    async def _ban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
+        user = await self.get_user(user, group_)
+        chat = await self.get_chat(group_)
 
-        bot_message.save()
-
-        return bot_message
+        await self.client.edit_permissions(chat.original_object, user.original_object, view_messages=False)
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def _create_chat_from_telegram_chat(self, original_chat: constants.TELEGRAM_CHAT) -> Chat | None:
-        chat_name = self._get_name_from_entity(original_chat)
+        chat_name = self._get_entity_name(original_chat)
         if isinstance(original_chat, constants.TELEGRAM_USER):
             group_id = None
             group_name = None
@@ -130,7 +130,7 @@ class TelegramBot(MultiBot[TelegramClient]):
         return self.User(
             platform=self.platform,
             id=original_user.id,
-            name=self._get_name_from_entity(original_user),
+            name=self._get_entity_name(original_user),
             is_admin=is_admin,
             is_bot=original_user.bot,
             original_object=original_user
@@ -138,7 +138,7 @@ class TelegramBot(MultiBot[TelegramClient]):
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def _get_author(self, original_message: constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE) -> User | None:
-        return await self._create_user_from_telegram_user(original_message.sender, original_message.chat.id)
+        return await self._create_user_from_telegram_user(await original_message.get_sender(), original_message.chat_id)
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def _get_button_pressed_text(self, event: constants.TELEGRAM_EVENT) -> str | None:
@@ -149,34 +149,50 @@ class TelegramBot(MultiBot[TelegramClient]):
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def _get_button_presser_user(self, event: constants.TELEGRAM_EVENT) -> User | None:
-        return await self._create_user_from_telegram_user(event.sender, event.chat.id)
+        return await self._get_author(event)
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def _get_chat(self, original_message: constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE) -> Chat | None:
-        return await self._create_chat_from_telegram_chat(original_message.chat)
+        return await self._create_chat_from_telegram_chat(await original_message.get_chat())
+
+    @return_if_first_empty('', exclude_self_types='TelegramBot', globals_=globals())
+    def _get_entity_name(self, entity: telethon.hints.EntityLike) -> str:
+        if isinstance(entity, constants.TELEGRAM_USER):
+            return entity.username or entity.first_name
+        elif isinstance(entity, constants.TELEGRAM_CHAT):
+            return entity.title
+
+        return ''
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def _get_mentions(self, original_message: constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE) -> list[User]:
+        if isinstance(original_message, constants.TELEGRAM_INLINE_EVENT):
+            return []
+
         mentions = OrderedSet()
-        try:
-            entities = original_message.entities or ()
-        except AttributeError:
-            return list(mentions)
+
+        if replied_message := await self._get_replied_message(original_message):
+            mentions.add(replied_message.author)
 
         text = await self._get_text(original_message)
         chat = await self._get_chat(original_message)
 
-        for entity in entities:
+        for entity in original_message.entities or ():
+            if not isinstance(entity, telethon.tl.types.MessageEntityMention):
+                continue
+
             try:
                 mentions.add(await self.get_user(text[entity.offset:entity.offset + entity.length], chat.group_id))
             except ValueError:
-                pass
+                if getattr(entity, 'user_id', None):
+                    # noinspection PyUnresolvedReferences
+                    mentions.add(await self.get_user(entity.user_id, chat.group_id))
 
         text = flanautils.remove_symbols(text, replace_with=' ')
         words = text.lower().split()
 
         for participant in await self.client.get_participants(chat.original_object):
-            user_name = self._get_name_from_entity(participant).lower()
+            user_name = self._get_entity_name(participant).lower()
             if user_name in words:
                 mentions.add(await self._create_user_from_telegram_user(participant, chat))
         if chat.is_private:
@@ -189,27 +205,20 @@ class TelegramBot(MultiBot[TelegramClient]):
     async def _get_message_id(self, original_message: constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE) -> int | None:
         return original_message.id
 
-    @return_if_first_empty('', exclude_self_types='TelegramBot', globals_=globals())
-    def _get_name_from_entity(self, entity: telethon.hints.EntityLike) -> str:
-        if isinstance(entity, telethon.types.User):
-            return entity.username or entity.first_name
-        elif isinstance(entity, (telethon.types.Channel, telethon.types.Chat)):
-            return entity.title
-
-        return ''
-
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def _get_original_message(self, event: constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE) -> constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE:
-        if isinstance(event, telethon.events.CallbackQuery.Event):
+        if isinstance(event, constants.TELEGRAM_NEW_MESSAGE_EVENT):
+            return event.message
+        elif isinstance(event, constants.TELEGRAM_BUTTON_EVENT):
             return await event.get_message()
         else:
-            return getattr(event, 'message', event)
+            return event
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def _get_replied_message(self, original_message: constants.TELEGRAM_EVENT | constants.TELEGRAM_MESSAGE) -> Message | None:
         try:
             return await self._get_message(await original_message.get_reply_message())
-        except AttributeError:
+        except (AttributeError, telethon.errors.rpcerrorlist.BotMethodInvalidError):
             pass
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
@@ -218,7 +227,7 @@ class TelegramBot(MultiBot[TelegramClient]):
 
     @staticmethod
     @return_if_first_empty
-    async def _prepare_media_to_send(media: Media, prefer_bytes=False, inline_=False) -> str | io.BytesIO | None:
+    async def _prepare_media_to_send(media: Media, prefer_bytes=False, is_inline=False) -> str | io.BytesIO | None:
         def url_file() -> str | None:
             if not media.url:
                 return
@@ -230,13 +239,20 @@ class TelegramBot(MultiBot[TelegramClient]):
 
         async def bytes_file() -> io.BytesIO | None:
             if not media.bytes_:
-                return
+                try:
+                    media.bytes_ = await flanautils.get_request(media.url)
+                except ResponseError:
+                    return
 
             bytes_ = media.bytes_
-            if inline_ and media.type_ is MediaType.AUDIO:
-                bytes_ = await flanautils.edit_metadata(bytes_, {'title': f'bot_media.{media.extension}'}, overwrite=False)
+            file_stem = media.title or 'bot_media'
+            if media.title or is_inline:
+                try:
+                    bytes_ = await flanautils.edit_metadata(bytes_, {'title': file_stem}, overwrite=False)
+                except FileNotFoundError:
+                    pass
             file_ = io.BytesIO(bytes_)
-            file_.name = f"{media.title or 'bot_media'}.{media.extension}"
+            file_.name = f"{file_stem}{f'.{media.extension}' if media.extension else ''}"
             return file_
 
         if prefer_bytes:
@@ -246,18 +262,23 @@ class TelegramBot(MultiBot[TelegramClient]):
 
         return file
 
+    async def _start_async(self):
+        await self.sign_in()
+        await self._on_ready()
+        await self.client.run_until_disconnected()
+
+    async def _unban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
+        user = await self.get_user(user, group_)
+        chat = await self.get_chat(group_)
+
+        await self.client.edit_permissions(chat.original_object, user.original_object)
+
     # ---------------------------------------------- #
     #                    HANDLERS                    #
     # ---------------------------------------------- #
     @find_message
     async def _on_inline_query_raw(self, message: Message):
-        self.inline_call_index += 1
-        inline_call_index = self.inline_call_index
-        await asyncio.sleep(constants.INLINE_DELAY_SECONDS)
-
-        if inline_call_index == self.inline_call_index:
-            self.inline_call_index = 0
-            await self._on_new_message_raw(message)
+        await super()._on_new_message_raw(message)
 
     async def _on_ready(self):
         self.platform = Platform.TELEGRAM
@@ -265,7 +286,7 @@ class TelegramBot(MultiBot[TelegramClient]):
         self.id = me.id
         self.name = me.username
         if self.user_client:
-            async with self.user_client:
+            async with use_user_client(self):
                 self.owner_id = (await self.user_client.get_me()).id
         await super()._on_ready()
 
@@ -282,53 +303,31 @@ class TelegramBot(MultiBot[TelegramClient]):
         except AttributeError:
             pass
 
-    # async def ban(self, user: int | str | User, chat: int | str | Chat | Message, seconds: int | datetime.timedelta = None):  # todo4 test en grupo de pruebas
-    # if isinstance(user, User):
-    #     user = user.original_object
-    # if isinstance(chat, Chat):
-    #     chat = chat.original_object
-    # if isinstance(seconds, int):
-    #     seconds = datetime.timedelta(seconds=seconds)
-    #
-    # rights = telethon.tl.types.ChatBannedRights(
-    #     until_date=datetime.datetime.now(datetime.timezone.utc) + seconds if seconds else None,
-    #     view_messages=True,
-    #     send_messages=True,
-    #     send_media=True,
-    #     send_stickers=True,
-    #     send_gifs=True,
-    #     send_games=True,
-    #     send_inline=True,
-    #     embed_links=True,
-    #     send_polls=True,
-    #     change_info=True,
-    #     invite_users=True,
-    #     pin_messages=True
-    # )
-    #
-    # await self.bot_client(telethon.tl.functions.channels.EditBannedRequest(chat, user, rights))
-
     @user_client
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def clear(self, n_messages: int, chat: int | str | Chat | Message):
         if n_messages > constants.DELETE_MESSAGE_LIMIT:
             raise LimitError('El máximo es 100.')
 
+        n_messages = int(n_messages)
         chat = await self.get_chat(chat)
-        n_messages += 1
 
-        async with self.user_client:
-            owner_user = await self._create_user_from_telegram_user(await self.user_client.get_me(), chat.group_id)
-            if owner_user not in await self.get_users(chat):
+        async with use_user_client(self):
+            user_client_user = await self._create_user_from_telegram_user(await self.user_client.get_me(), chat.group_id)
+            if await self.client.is_bot() and user_client_user not in await self.get_users(chat):
                 return
 
-            original_chat = await self.user_client.get_entity(chat.id)  # it's necessary
-            messages_to_delete = await self.user_client.get_messages(original_chat, n_messages)
-            await self.user_client.delete_messages(original_chat, messages_to_delete)
-            database_messages_to_delete_generator = self.Message.find({'platform': self.platform.value, 'chat': chat.object_id}, sort_keys=(('date', pymongo.DESCENDING),), limit=n_messages, lazy=True)
-            for database_message_to_delete in database_messages_to_delete_generator:
-                database_message_to_delete.is_deleted = True
-                database_message_to_delete.save()
+            if chat.is_group:
+                original_chat = await self.user_client.get_entity(chat.original_object)
+            else:
+                original_chat = await self.user_client.get_input_entity(self.name)
+
+            message_ids = [message.id async for message in self.user_client.iter_messages(original_chat, n_messages)]
+            await self.user_client.delete_messages(original_chat, message_ids)
+            deleted_messages_generator = self.Message.find({'platform': self.platform.value, 'chat': chat.object_id}, sort_keys=(('date', pymongo.DESCENDING),), limit=n_messages, lazy=True)
+            for deleted_message in deleted_messages_generator:
+                deleted_message.is_deleted = True
+                deleted_message.save()
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def delete_message(
@@ -347,7 +346,7 @@ class TelegramBot(MultiBot[TelegramClient]):
             await self.client.delete_messages(chat.original_object, message_to_delete.id)
 
         message_to_delete.is_deleted = True
-        message_to_delete.save()
+        message_to_delete.save(('is_deleted',))
 
     # noinspection PyTypeChecker
     def distribute_buttons(self, texts: Sequence[str], vertically=False) -> list[list[str]]:
@@ -362,21 +361,27 @@ class TelegramBot(MultiBot[TelegramClient]):
             case self.User() as user:
                 if user.original_object:
                     return await self._create_chat_from_telegram_chat(user.original_object)
-                chat_id = user.id
+                chat_id_or_name = user.id
             case self.Chat():
                 return chat
             case self.Message() as message:
                 return message.chat
-            case _ as chat_id:
+            case _ as chat_id_or_name:
                 pass
 
-        return await self._create_chat_from_telegram_chat(await self.client.get_entity(chat_id))
+        try:
+            return await self._create_chat_from_telegram_chat(await self.client.get_entity(chat_id_or_name))
+        except (ValueError, telethon.errors.rpcerrorlist.UsernameInvalidError):
+            pass
 
     async def get_me(self, group_: int | str | Chat = None):
         return await self._create_user_from_telegram_user(await self.client.get_me(), group_)
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
-    async def get_message(self, chat: int | str | User | Chat | Message, message: int | str | Message) -> Message | None:
+    async def get_message(self, message: int | str | Message, chat: int | str | User | Chat | Message) -> Message | None:
+        if not (chat := await self.get_chat(chat)):
+            return
+
         match message:
             case int(message_id):
                 pass
@@ -387,21 +392,49 @@ class TelegramBot(MultiBot[TelegramClient]):
             case _:
                 raise TypeError('bad arguments')
 
-        chat = await self.get_chat(chat)
-        return await self._get_message(await self.client.get_messages(chat.original_object, ids=message_id))
+        try:
+            with flanautils.suppress_low_level_stderr():
+                return await self._get_message(await self.client.get_messages(chat.original_object, ids=message_id))
+        except (TypeError, struct.error):
+            pass
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def get_user(self, user: int | str | User, group_: int | str | Chat | Message = None) -> User | None:
-        group_id = self.get_group_id(group_)
+        match user:
+            case self.User() if user.original_object:
+                original_user = user.original_object
+            case _:
+                match user:
+                    case self.User():
+                        user_id_or_name = user.id
+                    case user_id_or_name:
+                        pass
+                try:
+                    original_user = await self.client.get_entity(user_id_or_name)
+                except (ValueError, telethon.errors.rpcerrorlist.UsernameInvalidError):
+                    return
 
-        with flanautils.suppress_stderr():
-            return await self._create_user_from_telegram_user(await self.client.get_entity(user), group_id)
+        return await self._create_user_from_telegram_user(original_user, group_)
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def get_users(self, group_: int | str | Chat | Message) -> list[User]:
         chat = await self.get_chat(group_)
         return [await self._create_user_from_telegram_user(participant, chat.id) for participant in await self.client.get_participants(chat.original_object)]
 
+    async def make_mention(self, user: int | str | User, group_: int | str | Chat | Message = None) -> str:
+        if isinstance(user, int):
+            id = user
+            text = str(id)
+            prefix = ''
+        else:
+            if isinstance(user, str):
+                user = await self.get_user(user, group_)
+            id = user.id
+            text = user.name
+            prefix = '@' if user.original_object.username else ''
+
+        return f'<a href="tg://user?id={id}">{prefix}{text}</a>'
+
     @parse_arguments
     async def send(
         self,
@@ -416,7 +449,7 @@ class TelegramBot(MultiBot[TelegramClient]):
         data: dict = None,
         silent: bool = False,
         send_as_file: bool = None,
-        edit=False,
+        edit=False
     ) -> Message | None:
         file = await self._prepare_media_to_send(media)
         telegram_buttons = None
@@ -431,7 +464,8 @@ class TelegramBot(MultiBot[TelegramClient]):
 
         kwargs = {
             'file': file,
-            'parse_mode': 'html'
+            'parse_mode': 'html',
+            'supports_streaming': True
         }
 
         if message:
@@ -439,19 +473,18 @@ class TelegramBot(MultiBot[TelegramClient]):
                 word_matches = flanautils.cartesian_product_string_matching(message.text, constants.KEYWORDS['send_as_file'], min_score=constants.TELEGRAM_SEND_AS_FILE_MIN_SCORE)
                 send_as_file_score = sum(max(matches.values()) for text_word, matches in word_matches.items())
                 kwargs['force_document'] = bool(send_as_file_score)
+            else:
+                kwargs['force_document'] = send_as_file
 
             if message.is_inline:
                 if media:
-                    if 'inline_media' not in message.data:
-                        message.data['inline_media'] = []
-                    message.data['inline_media'].append(media)
+                    if 'inline_medias' not in message.data:
+                        message.data['inline_medias'] = []
+                    message.data['inline_medias'].append(media)
                 return
             elif edit:
                 if buttons is not None:
                     kwargs['buttons'] = telegram_buttons
-                    message.buttons_info.buttons = buttons
-                if buttons_key is not None:
-                    message.buttons_info.key = buttons_key
 
                 try:
                     message.original_object = await message.original_object.edit(text, **kwargs)
@@ -463,17 +496,7 @@ class TelegramBot(MultiBot[TelegramClient]):
                 ):
                     return
 
-                if data is None:
-                    if media is not None:
-                        message.data['media'] = media.content
-                else:
-                    if media is None:
-                        message.data = data
-                    else:
-                        message.data = {'media': media.content} | data
-                message.update_last_edit()
-                message.save()
-                return message
+                return self._update_message_attributes(message, media, buttons, chat, buttons_key, data, update_last_edit=True)
 
         match reply_to:
             case str():
@@ -482,32 +505,51 @@ class TelegramBot(MultiBot[TelegramClient]):
                 reply_to = message_to_reply.original_object
 
         with flanautils.suppress_stderr():
-            parse_retries = 1
-            while True:
+            attempts = 2
+            for attempt in range(attempts - 1, -1, -1):
                 try:
                     original_message = await self.client.send_message(chat.original_object, text, buttons=telegram_buttons, reply_to=reply_to, silent=silent, **kwargs)
-                except telethon.errors.rpcerrorlist.WebpageCurlFailedError:
-                    if media.bytes_:
-                        kwargs['file'] = await self._prepare_media_to_send(media, prefer_bytes=True)
+                except telethon.errors.VideoContentTypeInvalidError:
+                    if attempt:
+                        try:
+                            del kwargs['supports_streaming']
+                        except KeyError:
+                            pass
+                    else:
+                        raise
+                except telethon.errors.WebpageCurlFailedError:
+                    if bytes_ := await self._prepare_media_to_send(media, prefer_bytes=True):
+                        kwargs['file'] = bytes_
                     else:
                         raise
                 except ValueError as e:
-                    if 'parse' in str(e).lower() and parse_retries:
-                        del kwargs['parse_mode']
-                        parse_retries -= 1
+                    if 'parse' in str(e).lower() and attempt:
+                        try:
+                            del kwargs['parse_mode']
+                        except KeyError:
+                            pass
                     else:
                         raise
+                except telethon.errors.rpcerrorlist.MediaEmptyError:
+                    if (bytes_ := await self._prepare_media_to_send(media, prefer_bytes=True)) and isinstance(bytes_, bytes):
+                        kwargs['file'] = bytes_
+                    else:
+                        return
                 except (telethon.errors.rpcerrorlist.PeerIdInvalidError, telethon.errors.rpcerrorlist.UserIsBlockedError):
                     return
                 else:
                     break
 
-        return await self._create_bot_message_from_telegram_bot_message(original_message, media, chat, buttons, buttons_key, data)
+        original_message._sender = await self.client.get_me()
+        original_message._chat = chat.original_object
+        bot_message = await self._get_message(original_message)
+
+        return self._update_message_attributes(bot_message, media, buttons, chat, buttons_key, data)
 
     @inline
     async def send_inline_results(self, message: Message):
         async def create_result(media: Media, prefer_bytes=False) -> telethon.types.InputBotInlineResultPhoto | telethon.types.InputBotInlineResultDocument:
-            file = await self._prepare_media_to_send(media, prefer_bytes, inline_=True)
+            file = await self._prepare_media_to_send(media, prefer_bytes, is_inline=True)
             match media.type_:
                 case MediaType.IMAGE:
                     return message.original_event.builder.photo(file)
@@ -517,56 +559,49 @@ class TelegramBot(MultiBot[TelegramClient]):
         with flanautils.suppress_stderr():
             try:
                 try:
-                    await message.original_event.answer([await create_result(media) for media in message.data['inline_media']])
+                    await message.original_event.answer([await create_result(media) for media in message.data['inline_medias']])
                 except telethon.errors.rpcerrorlist.WebpageCurlFailedError:
-                    await message.original_event.answer([await create_result(media, prefer_bytes=True) for media in message.data['inline_media']])
-            except (KeyError, telethon.errors.rpcerrorlist.QueryIdInvalidError):
+                    await message.original_event.answer([await create_result(media, prefer_bytes=True) for media in message.data['inline_medias']])
+            except (AttributeError, KeyError, telethon.errors.rpcerrorlist.QueryIdInvalidError):
                 pass
 
-    def start(self):
-        async def start_():
+    async def sign_in(self):
+        if not self.bot_session and self.client:
+            print('----- Bot client -----')
             await self.client.connect()
+            await self.client.sign_in(bot_token=self.token)
+            print('Done.')
 
-            if not self.bot_session:
-                print('----- Bot client -----')
-                if not self.token:
-                    self.token = input('Enter a bot token: ').strip()
-                await self.client.sign_in(bot_token=self.token)
-                print('Done.')
-            if not self.user_session and self.user_client:
-                print('----- User client -----')
-                async with self.user_client:
-                    await self.user_client.sign_in(phone=self.phone)
-                print('Done.')
+        if not self.user_session and self.user_client:
+            print('----- User client -----')
+            await self.user_client.connect()
+            if not await self.user_client.is_user_authorized():
+                await self.user_client.sign_in(self.phone)
+                code = input('Enter the login code: ')
+                await self.user_client.sign_in(self.phone, code)
+            print('Done.')
 
-            await self._on_ready()
-            await self.client.run_until_disconnected()
+        if not self.client:
+            self.client = self.user_client
+
+        await self.client.connect()
+
+    @property
+    def string_sessions(self) -> dict[str, str] | Coroutine:
+        async def string_session_() -> dict[str, str]:
+            await self.sign_in()
+            return {
+                'bot_session': StringSession.save(self.client.session) if self.client else None,
+                'user_session': StringSession.save(self.user_client.session) if self.user_client else None
+            }
 
         try:
             asyncio.get_running_loop()
-            return start_()
         except RuntimeError:
-            asyncio.run(start_())
+            return asyncio.run(string_session_())
+        else:
+            return string_session_()
 
-    # async def unban(self, user: int | str | User, chat: int | str | Chat | Message):  # todo4 test en grupo de pruebas
-    # if isinstance(user, User):
-    #     user = user.original_object
-    # if isinstance(chat, Chat):
-    #     chat = chat.original_object
-    #
-    # rights = telethon.tl.types.ChatBannedRights(
-    #     view_messages=False,
-    #     send_messages=False,
-    #     send_media=False,
-    #     send_stickers=False,
-    #     send_gifs=False,
-    #     send_games=False,
-    #     send_inline=False,
-    #     embed_links=False,
-    #     send_polls=False,
-    #     change_info=False,
-    #     invite_users=False,
-    #     pin_messages=False
-    # )
-    #
-    # await self.bot_client(telethon.tl.functions.channels.EditBannedRequest(chat, user, rights))
+    async def typing(self, chat: int | str | User | Chat | Message) -> contextlib.AbstractAsyncContextManager:
+        chat = await self.get_chat(chat)
+        return self.client.action(chat.original_object, 'typing')
