@@ -8,7 +8,7 @@ from flanautils import FlanaBase
 from multibot.models.user import User
 
 
-@dataclass(eq=False)
+@dataclass
 class Button(FlanaBase):
     text: str = None
     is_checked: bool = False
@@ -17,7 +17,7 @@ class Button(FlanaBase):
         return bytes(self)
 
 
-@dataclass(eq=False)
+@dataclass
 class ButtonsInfo(FlanaBase):
     pressed_text: str = None
     presser_user: User = None
@@ -47,4 +47,4 @@ class ButtonsInfo(FlanaBase):
 
     @property
     def pressed_button(self) -> Button | None:
-        return self[self.pressed_text]
+        return self.find_button(self.pressed_text)
